//---------------------------------------------------------------------------
// Copyright (C) 2001 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//---------------------------------------------------------------------------
//
//  TODO.C - Link Layer functions required by general 1-Wire drive
//           implimentation.  Fill in the platform specific code.
//
//  Version: 3.00
//
//  History: 1.00 -> 1.01  Added function msDelay.
//           1.02 -> 1.03  Added function msGettick.
//           1.03 -> 2.00  Changed 'MLan' to 'ow'. Added support for
//                         multiple ports.
//           2.10 -> 3.00  Added owReadBitPower and owWriteBytePower
//

#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <termios.h>
#include <sys/time.h>
#include <time.h>
#ifdef LINUX
#include <linux/serial.h>
#endif
#include "ownet.h"

// exportable link-level functions
SMALLINT owTouchReset(int);
SMALLINT owTouchBit(int,SMALLINT);
SMALLINT owTouchByte(int,SMALLINT);
SMALLINT owWriteByte(int,SMALLINT);
SMALLINT owReadByte(int);
SMALLINT owSpeed(int,SMALLINT);
SMALLINT owLevel(int,SMALLINT);
SMALLINT owProgramPulse(int);
void msDelay(int);
long msGettick(void);
SMALLINT hasPowerDelivery(int);
SMALLINT hasOverDrive(int);
SMALLINT hasProgramPulse(int);
SMALLINT owWriteBytePower(int,SMALLINT);
SMALLINT owReadBitPower(int, SMALLINT);


extern int fd[MAX_PORTNUM];

//--------------------------------------------------------------------------
// Reset all of the devices on the 1-Wire Net and return the result.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
//
// Returns: TRUE(1):  presense pulse(s) detected, device(s) reset
//          FALSE(0): no presense pulses detected
//

SMALLINT owTouchReset(int portnum)
{
   fd_set		readset;
   struct timeval	wait_tm;
   int			stat;
   unsigned char		wbuff, c;

   int timeout = 5;
   int msec = 0;
   extern int global_msec, global_msec_max;
   extern struct termios term[MAX_PORTNUM];

   /* Flush the input and output buffers */
   tcflush( fd[portnum], TCIOFLUSH );
   
   /* 8 data bits */
   term[portnum].c_cflag |= CS8; //| CREAD | HUPCL | CLOCAL;
   
   cfsetispeed( &term[portnum], B9600 );	/* Set input speed to 115.2k	*/
   cfsetospeed( &term[portnum], B9600 );	/* Set output speed to 115.2k	*/
   
   if(tcsetattr(fd[portnum], TCSANOW, &term[portnum]) < 0 )
     {
	OWERROR(OWERROR_SYSTEM_RESOURCE_INIT_FAILED);
	printf("Reset: Error with tcsetattr 1: %s\n", strerror(errno));
	close(fd[portnum]);
	return FALSE;
     }
   
   /* Send the reset pulse */
   wbuff = 0xF0;
   write(fd[portnum], &wbuff, 1);
   
   /* Wait for a response or timeout */
   FD_ZERO(&readset);
   
   /* Timeout after 5 seconds */
   wait_tm.tv_usec = 0;
   wait_tm.tv_sec = timeout;
   
   c = 0;
   stat = 0;
   
   FD_SET(fd[portnum], &readset);
   
   /* Read byte if it doesn't timeout first */
   if (select( fd[portnum]+1, &readset, NULL, NULL, &wait_tm ) > 0 )
     {
	/* Is there something to read? */
	if (FD_ISSET(fd[portnum], &readset))
	  {
	     read(fd[portnum], &c, 1);
	     
	     /* If c is still zero, then it is a short to ground */
	     if (c == 0)
	       stat = FALSE;
	     
	     /* If the byte read is not what we sent, check for an error	*/
	     /* For now just return a 1 and discount any errors?? 		*/
	     if (c != 0xF0)
	       {  
		stat = TRUE;			/* Got a response of some type	*/
	       } else {
		  OWERROR(OWERROR_NO_DEVICES_ON_NET);
		  stat = FALSE;			/* No device responding		*/
	       }
	  }
     } else {
	
	stat = FALSE;				/* Timed out			*/
     }
   
   
   cfsetispeed(&term[portnum], B115200);	/* Set input speed to 115.2k	*/
   cfsetospeed(&term[portnum], B115200);	/* Set output speed to 115.2k	*/
   
   term[portnum].c_cflag |= CS6;		/* 6 data bits */
   
   if (tcsetattr(fd[portnum], TCSANOW, &term[portnum] ) < 0 )
     {
	OWERROR(OWERROR_SYSTEM_RESOURCE_INIT_FAILED);
	printf("Reset: Error with tcsetattr 2: %s\n", strerror(errno));
	close(fd[portnum]);
	return FALSE;
     }
   
   return stat;
}


int owTouchBits( int portnum, int timeout, int nbits, SMALLINT outch  )
{
   unsigned char c, inch, sendbit, mask;
   fd_set readset;
   struct timeval tv;
   int x;
   
   mask = 1;
   inch = 0;
   
   /* Flush the input and output buffers */
   tcflush( fd[portnum], TCIOFLUSH );
   
   /* Get first bit ready to be sent */
   sendbit = (outch & 0x01) ? 0xFF : 0x00;
   
   
   /* Send the bits/get the bits */
   for( x = 0; x < nbits; x++ )
     {
	printf("TouchBits: outch: %X, sendbit: %X, mask: %X\n", outch, sendbit, mask);
    write( fd[portnum], &sendbit, 1 );		/* Send the bit			*/
	
	mask <<= 1;				/* Next bit			*/
	inch >>= 1;
	
		 
	sendbit = (outch & mask) ? 0xFF : 0x00; /* Prepare bit */
	
	/* Get the incoming bit if there is one */
	
	/* Initialize readset */
	FD_ZERO(&readset);
	FD_SET(fd[portnum], &readset);
	
	/* Set timeout*/
	tv.tv_usec = 0;
	tv.tv_sec = timeout;
	
	/* Read byte if it doesn't timeout first */
	if( select( fd[portnum]+1, &readset, NULL, NULL, &tv ) > 0 )
	  {
	     /* Is there something to read? */
	     if( FD_ISSET( fd[portnum], &readset ) )
	       {
		  c = 0;
		  read( fd[portnum], &c, 1 );
		  //inch |= (c & 0x01) ? 0x80 : 0x00;
		  inch |= (c & 0x01);
	      printf("TouchBits: c: %X, inch: %X, bit: %i\n", c, inch, x);
		  } 
	     else 
	       {
		  /* Wicked?, some signal or something */
		  printf("b0rked\n");
	       }
	     
	  } else {
	     return 0xFF;
	  }  
     }
   
   if (nbits == 1 && inch == 0x80)
	   inch = 1;
   printf("TouchBits: inch: %X, nbits: %i\n", inch, nbits);
   return inch;
}

//--------------------------------------------------------------------------
// Send 1 bit of communication to the 1-Wire Net and return the
// result 1 bit read from the 1-Wire Net.  The parameter 'sendbit'
// least significant bit is used and the least significant bit
// of the result is the return bit.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
// 'sendbit'    - the least significant bit is the bit to send
//
// Returns: 0:   0 bit read from sendbit
//          1:   1 bit read from sendbit
//
SMALLINT owTouchBit(int portnum, SMALLINT sendbit)
{
   //unsigned char c = 0;
   unsigned char inbit = 0;
   fd_set readset;
   struct timeval tv;
 
   /* Flush the input and output buffers */
   tcflush( fd[portnum], TCIOFLUSH );
   
   /* Get the bit ready to be sent */
   sendbit = (sendbit & 0x01) ? 0xFF : 0x00;
     
   /* Send the bits/get the bits */
    write( fd[portnum], &sendbit, 1 );		/* Send the bit			*/

	/* Get the incoming bit if there is one */
	
	/* Initialize readset */
	FD_ZERO(&readset);
	FD_SET(fd[portnum], &readset);
	
	/* Set timeout*/
	tv.tv_usec = 0;
	tv.tv_sec = 5;
	
	/* Read byte if it doesn't timeout first */
	if( select( fd[portnum]+1, &readset, NULL, NULL, &tv ) > 0 )
	  {
	     /* Is there something to read? */
	     if( FD_ISSET( fd[portnum], &readset ) )
	       {
		  read( fd[portnum], &inbit, 1 );
		  //inch |= (c & 0x01) ? 0x80 : 0x00;
		  inbit &= 0x01;
	       } 
	     else 
	       {
		  /* Wicked?, some signal or something */
		  printf("b0rked\n");
	       }
	     
	  } else {
	     return 0xFF;
	  }  
 
   //printf("TouchBit: sendbit: %02X, inbit: %02X\n", sendbit, inbit);
   return inbit;
}

//--------------------------------------------------------------------------
// Send 8 bits of communication to the 1-Wire Net and return the
// result 8 bits read from the 1-Wire Net.  The parameter 'sendbyte'
// least significant 8 bits are used and the least significant 8 bits
// of the result is the return byte.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
// 'sendbyte'   - 8 bits to send (least significant byte)
//
// Returns:  8 bytes read from sendbyte
//
SMALLINT owTouchByte(int portnum, SMALLINT sendbyte)
{
	int n;
	SMALLINT sendbit;
	SMALLINT inbyte;
	
	int mask = 1;
	int tmp = 0;
	
	sendbit = (sendbyte & 0x01) ? 0xFF : 0x00;
	tmp = (owTouchBit(portnum, sendbit) & 0x01) ? 0x80 : 0x00;
	inbyte |= tmp;
	//printf("%1i", sendbyte & 0x01);
	//printf("%1i", tmp ? 0x01 : 0x00);
	
	for (n = 1; n < 8; n++)
	{
		
		mask<<=1;
		inbyte>>=1;
		//printf("%1i", (sendbyte & mask) ? 0x01 : 0x00);
		sendbit = (sendbyte & mask) ? 0xFF : 0x00;
	    tmp = (owTouchBit(portnum, sendbit) & 0x01) ? 0x80 : 0x00;
		inbyte |= tmp;
		//printf("inbyte: %02X, Mask: %02X\n", inbyte, mask);
		//printf("%1i", tmp ? 0x01 : 0x00);
	}
	//printf("TouchByte: sendbyte: %02X, inbyte: %02X\n", sendbyte, inbyte);
	return inbyte;
}

//--------------------------------------------------------------------------
// Send 8 bits of communication to the 1-Wire Net and verify that the
// 8 bits read from the 1-Wire Net is the same (write operation).
// The parameter 'sendbyte' least significant 8 bits are used.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
// 'sendbyte'   - 8 bits to send (least significant byte)
//
// Returns:  TRUE: bytes written and echo was the same
//           FALSE: echo was not the same
//
SMALLINT owWriteByte(int portnum, SMALLINT sendbyte)
{
   return (owTouchByte(portnum,sendbyte) == sendbyte) ? TRUE : FALSE;
}

//--------------------------------------------------------------------------
// Send 8 bits of read communication to the 1-Wire Net and and return the
// result 8 bits read from the 1-Wire Net.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
//
// Returns:  8 bytes read from 1-Wire Net
//
SMALLINT owReadByte(int portnum)
{
   return owTouchByte(portnum,0xFF);
}

//--------------------------------------------------------------------------
// Set the 1-Wire Net communucation speed.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
// 'new_speed'  - new speed defined as
//                MODE_NORMAL     0x00
//                MODE_OVERDRIVE  0x01
//
// Returns:  current 1-Wire Net speed
//
SMALLINT owSpeed(int portnum, SMALLINT new_speed)
{
   return 0;
}

//--------------------------------------------------------------------------
// Set the 1-Wire Net line level.  The values for NewLevel are
// as follows:
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
// 'new_level'  - new level defined as
//                MODE_NORMAL     0x00
//                MODE_STRONG5    0x02
//                MODE_PROGRAM    0x04
//                MODE_BREAK      0x08
//
// Returns:  current 1-Wire Net level
//
SMALLINT owLevel(int portnum, SMALLINT new_level)
{
   return 0;
}

//--------------------------------------------------------------------------
// This procedure creates a fixed 480 microseconds 12 volt pulse
// on the 1-Wire Net for programming EPROM iButtons.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
//
// Returns:  TRUE  successful
//           FALSE program voltage not available
//
SMALLINT owProgramPulse(int portnum)
{
   // add platform specific code here
   return FALSE;
}

//--------------------------------------------------------------------------
//  Description:
//     Delay for at least 'len' ms
//
void msDelay(int len)
{
   struct timespec s;              // Set aside memory space on the stack
   
   s.tv_sec = len / 1000;
   s.tv_nsec = (len - (s.tv_sec * 1000)) * 1000000;
   nanosleep(&s, NULL);
}

//--------------------------------------------------------------------------
// Get the current millisecond tick count.  Does not have to represent
// an actual time, it just needs to be an incrementing timer.
//
long msGettick(void)
{
   struct timezone tmzone;
   struct timeval  tmval;
   long ms;
   
   gettimeofday(&tmval,&tmzone);
   ms = (tmval.tv_sec & 0xFFFF) * 1000 + tmval.tv_usec / 1000;
   return ms;
}

//--------------------------------------------------------------------------
// Send 8 bits of communication to the 1-Wire Net and verify that the
// 8 bits read from the 1-Wire Net is the same (write operation).  
// The parameter 'sendbyte' least significant 8 bits are used.  After the
// 8 bits are sent change the level of the 1-Wire net.
//
// 'portnum'  - number 0 to MAX_PORTNUM-1.  This number was provided to
//              OpenCOM to indicate the port number.
// 'sendbyte' - 8 bits to send (least significant byte)
//
// Returns:  TRUE: bytes written and echo was the same
//           FALSE: echo was not the same 
//
SMALLINT owWriteBytePower(int portnum, SMALLINT sendbyte)
{
   // replace if platform has better implementation (faster response)
   if (!hasPowerDelivery(portnum))
     return FALSE;
   
   if(owTouchByte(portnum,sendbyte) != sendbyte)
     return FALSE;
   
   if(owLevel(portnum,MODE_STRONG5) != MODE_STRONG5)
     return FALSE;
   
   return TRUE;
}

//--------------------------------------------------------------------------
// Send 1 bit of communication to the 1-Wire Net and verify that the
// response matches the 'applyPowerResponse' bit and apply power delivery
// to the 1-Wire net.  Note that some implementations may apply the power
// first and then turn it off if the response is incorrect.
//
// 'portnum'  - number 0 to MAX_PORTNUM-1.  This number was provided to
//              OpenCOM to indicate the port number.
// 'applyPowerResponse' - 1 bit response to check, if correct then start
//                        power delivery 
//
// Returns:  TRUE: bit written and response correct, strong pullup now on
//           FALSE: response incorrect
//
SMALLINT owReadBitPower(int portnum, SMALLINT applyPowerResponse)
{
   // replace if platform has better implementation (faster response)
   if (!hasPowerDelivery(portnum))
     return FALSE;
   
   if(owTouchBit(portnum,0x01) != applyPowerResponse)
     return FALSE;
   
   if(owLevel(portnum,MODE_STRONG5) != MODE_STRONG5)
     return FALSE;
   
   return TRUE;
}

//--------------------------------------------------------------------------
// This procedure indicates wether the adapter can deliver power.
//
// 'portnum'  - number 0 to MAX_PORTNUM-1.  This number was provided to
//              OpenCOM to indicate the port number.
//
// Returns:  TRUE  if adapter is capable of delivering power. 
//
SMALLINT hasPowerDelivery(int portnum)
{
   return TRUE;
}

//--------------------------------------------------------------------------
// This procedure indicates wether the adapter can deliver power.
//
// 'portnum'  - number 0 to MAX_PORTNUM-1.  This number was provided to
//              OpenCOM to indicate the port number.
//
// Returns:  TRUE  if adapter is capable of over drive. 
//
SMALLINT hasOverDrive(int portnum)
{
   // Passive adapter dosen't suport OverDrive.
   return FALSE;
}
//--------------------------------------------------------------------------
// This procedure creates a fixed 480 microseconds 12 volt pulse 
// on the 1-Wire Net for programming EPROM iButtons.
//
// 'portnum'  - number 0 to MAX_PORTNUM-1.  This number was provided to
//              OpenCOM to indicate the port number.
//
// Returns:  TRUE  program volatage available
//           FALSE program voltage not available  
SMALLINT hasProgramPulse(int portnum)
{
   return FALSE;
}
