//---------------------------------------------------------------------------
// Copyright (C) 1999 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//---------------------------------------------------------------------------
//
//  todoses.C - Acquire and release a Session Todo for general 1-Wire Net
//              library
//
//  Version: 2.00
//           1.03 -> 2.00  Changed 'MLan' to 'ow'. Added support for
//                         multiple ports.
//

#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include "ownet.h"

// local function prototypes
SMALLINT owAcquire(int,char *);
void     owRelease(int);


int fd[MAX_PORTNUM];
struct termios term[MAX_PORTNUM];
struct termios term_orig[MAX_PORTNUM];

//---------------------------------------------------------------------------
// Attempt to acquire a 1-Wire net
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
// 'port_zstr'  - zero terminated port name.
//
// Returns: TRUE - success, port opened
//
SMALLINT owAcquire(int portnum, char *port_zstr)
{  
   
   /* Open the serial port */
   if ((fd[portnum] = open(port_zstr, O_RDWR)) == -1)
     {
	OWERROR(OWERROR_GET_SYSTEM_RESOURCE_FAILED);

	return FALSE;
     }
   
   /* Get device settings */
   if(tcgetattr(fd[portnum], &term[portnum] ) < 0 )
     {
	OWERROR(OWERROR_SYSTEM_RESOURCE_INIT_FAILED);
	close(fd[portnum]);
	
	return FALSE;
     }
   
   term_orig[portnum] = term[portnum];
   
   /* Reset all settings */
   term[portnum].c_lflag = 0;
   term[portnum].c_iflag = 0;
   term[portnum].c_oflag = 0;
   
   /* 1 byte at a time, no timer */
   term[portnum].c_cc[VMIN] = 1;  
   term[portnum].c_cc[VTIME] = 0;
   
   
   /* 6 data bits, Receiver enabled, Hangup, Dont change "owner" */
   term[portnum].c_cflag |= CS6 | CREAD | HUPCL | CLOCAL;
   
   cfsetispeed(&term[portnum], B115200);       /* Set input speed to 115.2k    */
   cfsetospeed(&term[portnum], B115200);       /* Set output speed to 115.2k   */
   
   if(tcsetattr(fd[portnum], TCSANOW, &term[portnum]) < 0 )
     {
	OWERROR(OWERROR_SYSTEM_RESOURCE_INIT_FAILED);
	close(fd[portnum]);
	
	return FALSE;
     }
   
   
   /* Flush the input and output buffers */
   tcflush(fd[portnum], TCIOFLUSH);
   
   return TRUE;
}

//---------------------------------------------------------------------------
// Release the previously acquired a 1-Wire net.
//
// 'portnum'    - number 0 to MAX_PORTNUM-1.  This number is provided to
//                indicate the symbolic port number.
//
void owRelease(int portnum)
{
   if(tcsetattr(fd[portnum], TCSANOW, &term_orig[portnum]) < 0 )
     {
	OWERROR(OWERROR_SYSTEM_RESOURCE_INIT_FAILED);
	close(fd[portnum]);
     }
   
   if (close(fd[portnum]) < 0)
     {
	OWERROR(OWERROR_SYSTEM_RESOURCE_INIT_FAILED);
     }
}


